class Timer(object):
    def __init__(self):
        self.id = ""
        self.name = "timer"
        self.measure_time = ""
        self.measure_day = ""
        self.rules = []
        self.status = "connected"
        self.color = "green"
        self.expiration = "no"

    def device_mapping(self, device):
        self.id = device["id"]
        self.name = device["name"]
        self.rules = device["rules"]



from .AlertConsequentDTO import AlertConsequent
import smtplib
from email.mime.text import MIMEText


def email_connection(email_user, email_password):
    print("email connection")
    session = smtplib.SMTP("smtp.gmail.com", 587)
    session.starttls()
    session.login(email_user, email_password)
    return session


class AlertConsequentFunction(object):
    def __init__(self, redis):
        self.r = redis
        self.email_user = "raspberrypi.sugherotorto"
        self.email_password = "giomartagio"

    def get_consequent(self, user_id, rule_id, device_id):
        try:
            consequent = AlertConsequent()
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_consequents:" + device_id
            consequent.device_id = device_id
            consequent.device_name = self.r.get("device:" + device_id + ":name")
            consequent.message = self.r.get(key_pattern + ":message")
            consequent.delay = self.r.get(key_pattern + ":delay")
            consequent.order = self.r.get(key_pattern + ":order")
            return consequent
        except Exception as error:
            print(repr(error))
            return "error"

    def get_consequent_slim(self, user_id, rule_id, device_id):
        try:
            consequent = AlertConsequent()
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_consequents:" + device_id
            consequent.device_id = device_id
            consequent.device_name = self.r.get("device:" + device_id + ":name")
            consequent.order = self.r.get(key_pattern + ":order")
            consequent.delay = self.r.get(key_pattern + ":delay")
            return consequent
        except Exception as error:
            print(repr(error))
            return "error"

    def delete_consequent(self, user_id, rule_id, device_id):
        try:
            self.r.lrem("device:" + device_id + ":rules", rule_id)
            self.r.lrem("user:" + user_id + ":rule:" + rule_id + ":device_consequents", device_id)
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_consequents:" + device_id
            self.r.delete(key_pattern + ":message")
            self.r.delete(key_pattern + ":delay")
            self.r.delete(key_pattern + ":order")
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"

    def add_consequent(self, user_id, rule_id, device_id):
        try:
            device_consequents = self.r.lrange("user:" + user_id + ":rule:" + rule_id + ":device_consequents")
            result = "false"
            if device_id not in device_consequents:
                consequent = AlertConsequent()
                consequent.order = str(len(device_consequents))
                self.r.rpush("device:" + device_id + ":rules", rule_id)
                self.r.rpush("user:" + user_id + ":rule:" + rule_id + ":device_consequents", device_id)
                key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_consequents:" + device_id
                self.r.set(key_pattern + ":message", consequent.message)
                self.r.set(key_pattern + ":delay", consequent.delay)
                self.r.set(key_pattern + ":order", consequent.order)
                result = "true"
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def update_consequent(self, user_id, rule_id, new_consequent):
        try:
            consequent = AlertConsequent()
            consequent.consequent_mapping(new_consequent)
            device_consequents = self.r.lrange("user:" + user_id + ":rule:" + rule_id + ":device_consequents")
            result = "false"
            if consequent.device_id in device_consequents:
                key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_consequents:" + consequent.device_id
                self.r.set(key_pattern + ":message", consequent.message)
                self.r.set(key_pattern + ":delay", consequent.delay)
                self.r.set(key_pattern + ":order", consequent.order)
                result = "true"
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def alert_evaluation(self, user_id, rule_id):
        if self.r.get("user:" + user_id + ":rule:" + rule_id + ":evaluation") == "true":
            alert_id = "alert" + user_id
            sendto = self.r.lrange("device:" + alert_id + ":email_list")
            if len(sendto) > 0:
                alert = email_connection(self.email_user, self.email_password)
                rule_name = self.r.get("user:" + user_id + ":rule:" + rule_id + ":name")
                sender = "raspberrypi.sugherotorto@gmail.com"
                content = """Alert for rule name {}""".format(rule_name)
                msg = MIMEText(content, 'plain')
                msg['Subject'] = "ALERT YOURULE"
                msg['From'] = sender
                alert.sendmail(sender, sendto, msg.as_string())
                alert.quit()
                print("send alert email")



from .WaterLevelDTO import WaterLevel
from .WaterLevelAntecedentFunctions import WaterLevelAntecedentFunction
from datetime import datetime
from ..DeviceEvaluationDTO import DeviceEvaluation


class WaterLevelFunction(object):
    def __init__(self, redis):
        self.r = redis
        self.waterlevel_antecedent_functions = WaterLevelAntecedentFunction(redis)

    def register(self, user_id, device_id):
        try:
            result = "false"
            key_pattern = "device:" + device_id
            if self.r.exists(key_pattern + ":name") == 0:
                self.r.rpush("user:" + user_id + ":sensors", device_id)
                device = WaterLevel()
                device.id = device_id
                device_id_keys = self.r.lrange("user:" + user_id + ":sensors")
                device.name = "WATERLEVEL " + str(len(device_id_keys))
                self.r.set(key_pattern + ":name", device.name)
                self.r.set(key_pattern + ":user_id", user_id)
                self.r.set(key_pattern + ":measure", device.measure)
                self.r.set(key_pattern + ":absolute_measure", device.absolute_measure)
                self.r.set(key_pattern + ":setting:error", device.setting_error)
                self.r.set(key_pattern + ":setting:max", device.setting_max)
                self.r.set(key_pattern + ":max_measure", device.max_measure)
                self.r.set(key_pattern + ":max_measure_time", device.max_measure_time)
                self.r.set(key_pattern + ":max_measure_date", device.max_measure_date)
                self.r.set(key_pattern + ":min_measure", device.min_measure)
                self.r.set(key_pattern + ":min_measure_time", device.min_measure_time)
                self.r.set(key_pattern + ":min_measure_date", device.min_measure_date)
                self.r.set(key_pattern + ":expiration", device.expiration)
                result = device
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def get_device(self, user_id, device_id):
        try:
            key_pattern = "device:" + device_id
            dto = WaterLevel()
            dto.id = device_id
            dto.name = self.r.get(key_pattern + ":name")
            dto.setting_max = self.r.get(key_pattern + ":setting:max")
            dto.setting_error = self.r.get(key_pattern + ":setting:error")
            dto.max_measure = self.r.get(key_pattern + ":max_measure")
            dto.max_measure_time = self.r.get(key_pattern + ":max_measure_time")
            dto.min_measure = self.r.get(key_pattern + ":min_measure")
            dto.min_measure_time = self.r.get(key_pattern + ":min_measure_time")
            dto.expiration = self.r.get(key_pattern + ":expiration")
            if self.r.exists(key_pattern + ":rules") == 1:
                rules_id = self.r.lrange(key_pattern + ":rules")
                for rule_id in rules_id:
                    rule_name = self.r.get("user:" + user_id + ":rule:" + rule_id + ":name")
                    dto.rules.append({"id": rule_id, "name": rule_name})
            if self.r.exists(key_pattern + ":measure") == 1:
                measure = self.r.get(key_pattern + ":measure")
                if measure == "-":
                    dto.measure = measure
                    dto.absolute_measure = measure
                    dto.color = "yellow"
                    dto.status = "initialization"
                else:
                    dto.absolute_measure = self.r.get(key_pattern + ":absolute_measure")
                    relative_measure = float(dto.absolute_measure) - float(dto.setting_error)
                    dto.measure = str(round((1 - (relative_measure / float(dto.setting_max))) * 100.0))
                    dto.color = "green"
                    dto.status = "connected"
            return dto
        except Exception as error:
            print(repr(error))
            return "error"

    def update_device(self, new_device):
        try:
            dto = WaterLevel()
            dto.device_mapping(new_device)
            key_pattern = "device:" + dto.id
            self.r.set(key_pattern + ":name", dto.name)
            self.r.set(key_pattern + ":setting:max", dto.setting_max)
            self.r.set(key_pattern + ":setting:error", dto.setting_error)
            self.r.set(key_pattern + ":measure", dto.measure)
            self.r.set(key_pattern + ":expiration", dto.expiration)
            return dto
        except Exception as error:
            print(repr(error))
            return "error"

    def delete_device(self, user_id, device_id):
        try:
            self.r.lrem("user:" + user_id + ":sensors", device_id)
            key_pattern = "device:" + device_id
            self.r.delete(key_pattern + ":name")
            self.r.delete(key_pattern + ":user_id")
            self.r.delete(key_pattern + ":measure")
            self.r.delete(key_pattern + ":absolute_measure")
            self.r.delete(key_pattern + ":setting:max")
            self.r.delete(key_pattern + ":setting:error")
            self.r.delete(key_pattern + ":max_measure")
            self.r.delete(key_pattern + ":max_measure_time")
            self.r.delete(key_pattern + ":min_measure")
            self.r.delete(key_pattern + ":min_measure_time")
            if self.r.exists(key_pattern + ":rules") == 1:
                rules = self.r.lrange(key_pattern + ":rules")
                for rule_id in rules:
                    self.waterlevel_antecedent_functions.delete_antecedent(user_id, rule_id, device_id)
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"

    def measure_evaluation(self, device_id, absolute_measure):
        key_pattern = "device:" + device_id
        max_measure = int(self.r.get(key_pattern + ":setting:max"))
        error_setting = int(self.r.get(key_pattern + ":setting:error"))
        scaled_measure = float(absolute_measure) - float(error_setting)
        relative_measure = str(round((1 - (scaled_measure / float(max_measure))) * 100.0))
        expiration = int(self.r.get(key_pattern + ":expiration")) + 2
        self.r.setex(key_pattern + ":measure", expiration, relative_measure)
        self.r.setex(key_pattern + ":absolute_measure", expiration, absolute_measure)
        self.device_max_measure_range(device_id, absolute_measure)
        self.device_min_measure_range(device_id, absolute_measure)
        return relative_measure

    def device_max_measure_range(self, device_id, measure):
        key_pattern = "device:" + device_id
        time_str = datetime.now().strftime("%H:%M")
        date_str = datetime.now().strftime("%d/%m/%Y")
        if self.r.exists(key_pattern + ":max_measure"):
            max_measure = self.r.get(key_pattern + ":max_measure")
            if max_measure != "-":
                if float(measure) > float(max_measure):
                    self.r.set(key_pattern + ":max_measure", measure)
                    self.r.set(key_pattern + ":max_measure_time", time_str)
                    self.r.set(key_pattern + ":max_measure_date", date_str)
            else:
                self.r.set(key_pattern + ":max_measure", measure)
                self.r.set(key_pattern + ":max_measure_time", time_str)
                self.r.set(key_pattern + ":max_measure_date", date_str)
        else:
            self.r.set(key_pattern + ":max_measure", measure)
            self.r.set(key_pattern + ":max_measure_time", time_str)
            self.r.set(key_pattern + ":max_measure_date", date_str)

    def device_min_measure_range(self, device_id, measure):
        key_pattern = "device:" + device_id
        time_str = datetime.now().strftime("%H:%M")
        date_str = datetime.now().strftime("%d/%m/%Y")
        if self.r.exists(key_pattern + ":min_measure"):
            min_measure = self.r.get(key_pattern + ":min_measure")
            if min_measure != "-":
                if float(measure) < float(min_measure):
                    self.r.set(key_pattern + ":min_measure", measure)
                    self.r.set(key_pattern + ":min_measure_time", time_str)
                    self.r.set(key_pattern + ":min_measure_date", date_str)
            else:
                self.r.set(key_pattern + ":min_measure", measure)
                self.r.set(key_pattern + ":min_measure_time", time_str)
                self.r.set(key_pattern + ":min_measure_date", date_str)
        else:
            self.r.set(key_pattern + ":min_measure", measure)
            self.r.set(key_pattern + ":min_measure_time", time_str)
            self.r.set(key_pattern + ":min_measure_date", date_str)

    def device_evaluation(self, device_id, measure):
        output = DeviceEvaluation()
        key_pattern = "device:" + device_id
        if self.r.exists(key_pattern + ":user_id") == 1:
            output.user_id = self.r.get("device:" + device_id + ":user_id")
            output.measure = self.measure_evaluation(device_id, measure)
            output.device_id = device_id
            output.type = "antecedent"
            key_pattern = "device:" + device_id
            if self.r.exists(key_pattern + ":rules"):
                output.rules = self.r.lrange(key_pattern + ":rules")
        return output



from datetime import datetime


class RuleFunction(object):
    def __init__(self, redis):
        self.r = redis

    def rule_evaluation(self, user_id, rule_id):
        pattern_key = "user:" + user_id + ":rule:" + rule_id
        trigger = "false"
        if self.r.exists(pattern_key + ":name") == 1:
            old_evaluation = self.r.get(pattern_key + ":evaluation")
            antecedent_keys = self.r.scan(pattern_key + ":rule_antecedents:*:evaluation")
            new_evaluation = "false"
            consequents_status = self.check_consequents_status(user_id, rule_id)
            if consequents_status == "false":
                new_evaluation = "false"
            else:
                if len(antecedent_keys) > 0:
                    new_evaluation = self.check_antecedent_evaluation(antecedent_keys)
            if old_evaluation != new_evaluation:
                self.r.set(pattern_key + ":evaluation", new_evaluation)
                self.update_evaluation_timestamp(pattern_key, new_evaluation)
                trigger = "true"
        return trigger

    def check_antecedent_evaluation(self, antecedent_keys):
        new_evaluation = "true"
        for key in antecedent_keys:
            evaluation = self.r.get(key)
            if evaluation == "false":
                new_evaluation = "false"
                break
        return new_evaluation

    def check_consequents_status(self, user_id, rule_id):
        key_pattern = "user:" + user_id + ":rule:" + rule_id
        device_consequents = self.r.lrange(key_pattern + ":device_consequents")
        consequents_status = "true"
        for device_id in device_consequents:
            if "alert" not in device_id and self.r.exists("device:" + device_id + ":measure") == 0:
                consequents_status = "false"
                break
        return consequents_status

    def update_evaluation_timestamp(self, pattern_key, evaluation):
        time_str = datetime.now().strftime("%%H:%M:%S")
        date_str = datetime.now().strftime("%d/%m/%Y")
        if evaluation == "true":
            self.r.set(pattern_key + ":last_time_on", time_str)
            self.r.set(pattern_key + ":last_date_on", date_str)
        else:
            self.r.set(pattern_key + ":last_time_off", time_str)
            self.r.set(pattern_key + ":last_date_off", date_str)



from .SwitchDTO import Switch
from .SwitchConsequentFunctions import SwitchConsequentFunction
from ..DeviceEvaluationDTO import DeviceEvaluation
from datetime import datetime


class SwitchFunction(object):
    def __init__(self, redis):
        self.r = redis
        self.switch_consequent_functions = SwitchConsequentFunction(redis)

    def register(self, user_id, device_id):
        try:
            result = "false"
            key_pattern = "device:" + device_id
            if self.r.exists(key_pattern + ":name") == 0:
                self.r.rpush("user:" + user_id + ":switches", device_id)
                device_id_keys = self.r.lrange("user:" + user_id + ":switches")
                device = Switch()
                device.id = device_id
                device.name = "SWITCH " + str(len(device_id_keys))
                current_time_str = datetime.now().strftime("%H:%M")
                current_day = str(datetime.today().weekday())
                self.r.set(key_pattern + ":name", device.name)
                self.r.set(key_pattern + ":user_id", user_id)
                self.r.set(key_pattern + ":measure", device.measure)
                self.r.set(key_pattern + ":last_date_on", current_day)
                self.r.set(key_pattern + ":last_date_off", current_day)
                self.r.set(key_pattern + ":last_time_on", current_time_str)
                self.r.set(key_pattern + ":last_time_off", current_time_str)
                self.r.set(key_pattern + ":automatic", device.automatic)
                self.r.set(key_pattern + ":manual_measure", device.manual_measure)
                self.r.set(key_pattern + ":expiration", device.expiration)
                result = device
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def get_device(self, user_id, device_id):
        try:
            key_pattern = "device:" + device_id
            dto = Switch()
            dto.id = device_id
            dto.name = self.r.get(key_pattern + ":name")
            dto.automatic = self.r.get(key_pattern + ":automatic")
            dto.manual_measure = self.r.get(key_pattern + ":manual_measure")
            dto.last_date_on = self.r.get(key_pattern + ":last_date_on")
            dto.last_date_off = self.r.get(key_pattern + ":last_date_off")
            dto.last_time_on = self.r.get(key_pattern + ":last_time_on")
            dto.last_time_off = self.r.get(key_pattern + ":last_time_off")
            dto.expiration = self.r.get(key_pattern + "expiration")
            if self.r.exists(key_pattern + ":rules") == 1:
                rules_id = self.r.lrange(key_pattern + ":rules")
                for rule_id in rules_id:
                    rule_name = self.r.get("user:" + user_id + ":rule:" + rule_id + ":name")
                    dto.rules.append({"id": rule_id, "name": rule_name})
            if self.r.exists(key_pattern + ":measure") == 1:
                measure = self.r.get(key_pattern + ":measure")
                if measure == "-":
                    dto.measure = measure
                    dto.color = "yellow"
                    dto.status = "initialization"
                else:
                    dto.measure = measure
                    dto.color = "green"
                    dto.status = "connected"
            return dto
        except Exception as error:
            print(repr(error))
            return "error"

    def update_device(self, new_device):
        try:
            dto = Switch()
            dto.device_mapping(new_device)
            key_pattern = "device:" + dto.id
            self.r.set(key_pattern + ":name", dto.name)
            self.r.set(key_pattern + ":automatic", dto.automatic)
            self.r.set(key_pattern + ":manual_measure", dto.manual_measure)
            self.r.set(key_pattern + ":expiration", dto.expiration)
            return dto
        except Exception as error:
            print(repr(error))
            return "error"

    def delete_device(self, user_id, device_id):
        try:
            self.r.lrem("user:" + user_id + ":switches", device_id)
            key_pattern = "device:" + device_id
            self.r.delete(key_pattern + ":name")
            self.r.delete(key_pattern + ":user_id")
            self.r.delete(key_pattern + ":measure")
            self.r.delete(key_pattern + ":manual_measure")
            self.r.delete(key_pattern + ":automatic")
            self.r.delete(key_pattern + ":last_date_on")
            self.r.delete(key_pattern + ":last_date_off")
            self.r.delete(key_pattern + ":last_time_on")
            self.r.delete(key_pattern + ":last_time_off")
            if self.r.exists(key_pattern + ":rules") == 1:
                rules = self.r.lrange(key_pattern + ":rules")
                for rule_id in rules:
                    self.switch_consequent_functions.delete_consequent(user_id, rule_id, device_id)
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"

    def device_evaluation(self, device_id, measure):
        output = DeviceEvaluation()
        key_pattern = "device:" + device_id
        if self.r.exists(key_pattern + ":user_id") == 1:
            expiration = int(self.r.get(key_pattern + ":expiration")) + 2
            self.r.setex(key_pattern + ":measure", expiration, measure)
            self.last_time_status_update(device_id, measure)
            user_id = self.r.get(key_pattern + ":user_id")
            evaluated_measure = self.measure_evaluation(user_id, device_id)
            if evaluated_measure != measure:
                output.user_id = user_id
                output.device_id = device_id
                output.type = "switch"
                output.measure = evaluated_measure
        return output

    def measure_evaluation(self, user_id, device_id):
        key_pattern = "device:" + device_id
        automatic = self.r.get(key_pattern + ":automatic")
        if automatic == "true":
            rules = self.r.lrange(key_pattern + ":rules")
            measure = "off"
            for rule in rules:
                if self.r.get("user:" + user_id + ":rule:" + rule + ":evaluation") == "true":
                    measure = "on"
                    break
        else:
            measure = self.r.get(key_pattern + ":manual_measure")
        return measure

    def last_time_status_update(self, device_id, measure):
        time_str = datetime.now().strftime("%H:%M:%S")
        date_str = datetime.now().strftime("%d/%m/%Y")
        key_pattern = "device:" + device_id
        if measure == "on":
            self.r.set(key_pattern + ":last_time_on", time_str)
            self.r.set(key_pattern + ":last_date_on", date_str)
        else:
            self.r.set(key_pattern + ":last_time_off", time_str)
            self.r.set(key_pattern + ":last_date_off", date_str)



from .SwitchAntecedentDTO import SwitchAntecedent
from datetime import datetime


class SwitchAntecedentFunction(object):
    def __init__(self, redis):
        self.r = redis

    def get_antecedent(self, user_id, rule_id, device_id):
        try:
            antecedent = SwitchAntecedent()
            rule_key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            device_key_pattern = "device:" + device_id
            antecedent.device_id = device_id
            antecedent.device_name = self.r.get(device_key_pattern + ":name")
            antecedent.last_time_on = self.r.get(device_key_pattern + ":last_time_on")
            antecedent.last_date_on = self.r.get(device_key_pattern + ":last_date_on")
            antecedent.last_time_off = self.r.get(device_key_pattern + ":last_time_off")
            antecedent.last_date_off = self.r.get(device_key_pattern + ":last_date_off")
            antecedent.evaluation = self.r.get(rule_key_pattern + ":evaluation")
            antecedent.time_start_value = self.r.get(rule_key_pattern + ":time_start_value")
            antecedent.time_stop_value = self.r.get(rule_key_pattern + ":time_stop_value")
            antecedent.date_start_value = self.r.get(rule_key_pattern + ":date_start_value")
            antecedent.date_stop_value = self.r.get(rule_key_pattern + ":date_stop_value")
            return antecedent
        except Exception as error:
            print(repr(error))
            return "error"

    def get_antecedent_slim(self, user_id, rule_id, device_id):
        try:
            antecedent = SwitchAntecedent()
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            antecedent.device_id = device_id
            antecedent.device_name = self.r.get("device:" + device_id + ":name")
            antecedent.evaluation = self.r.get(key_pattern + ":evaluation")
            return antecedent
        except Exception as error:
            print(repr(error))
            return "error"

    def delete_antecedent(self, user_id, rule_id, device_id):
        try:
            self.r.lrem("device:" + device_id + ":rules", rule_id)
            self.r.lrem("user:" + user_id + ":rule:" + rule_id + ":device_antecedents", device_id)
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            self.r.delete(key_pattern + ":evaluation")
            self.r.delete(key_pattern + ":last_time_on")
            self.r.delete(key_pattern + ":last_time_off")
            self.r.delete(key_pattern + ":last_date_on")
            self.r.delete(key_pattern + ":last_date_off")
            self.r.delete(key_pattern + ":time_start_value")
            self.r.delete(key_pattern + ":time_stop_value")
            self.r.delete(key_pattern + ":date_start_value")
            self.r.delete(key_pattern + ":date_stop_value")
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"

    def add_antecedent(self, user_id, rule_id, device_id):
        try:
            device_antecedents = self.r.lrange("user:" + user_id + ":rule:" + rule_id + ":device_antecedents")
            device_consequents = self.r.lrange("user:" + user_id + ":rule:" + rule_id + ":device_consequents")
            result = "false"
            if device_id not in device_antecedents and device_id in device_consequents:
                self.r.rpush("user:" + user_id + ":rule:" + rule_id + ":device_antecedents", device_id)
                key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
                antecedent = SwitchAntecedent()
                self.r.set(key_pattern + ":evaluation", antecedent.evaluation)
                self.r.set(key_pattern + ":time_start_value", antecedent.time_start_value)
                self.r.set(key_pattern + ":time_stop_value", antecedent.time_stop_value)
                self.r.set(key_pattern + ":date_start_value", antecedent.date_start_value)
                self.r.set(key_pattern + ":date_stop_value", antecedent.date_stop_value)
                result = "true"
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def update_antecedent(self, user_id, rule_id, new_antecedent):
        try:
            antecedent = SwitchAntecedent()
            antecedent.antecedent_mapping(new_antecedent)
            device_antecedents = self.r.lrange("user:" + user_id + ":rule:" + rule_id + ":device_antecedents")
            result = "false"
            if antecedent.device_id in device_antecedents:
                key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + antecedent.device_id
                self.r.set(key_pattern + ":time_start_value", antecedent.time_start_value)
                self.r.set(key_pattern + ":time_stop_value", antecedent.time_stop_value)
                self.r.set(key_pattern + ":date_start_value", antecedent.date_start_value)
                self.r.set(key_pattern + ":date_stop_value", antecedent.date_stop_value)
                result = "true"
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def antecedent_evaluation(self, user_id, device_id, rule_id):
        try:
            evaluation = self.last_time_evaluation(user_id, device_id, rule_id)
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            old_evaluation = self.r.get(key_pattern + ":evaluation")
            trigger = "false"
            if evaluation != old_evaluation:
                self.r.set(key_pattern + ":evaluation", evaluation)
                trigger = "true"
            return trigger
        except Exception as error:
            print(repr(error))
            return "error"

    def last_time_evaluation(self, user_id, device_id, rule_id):
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        device_key_pattern = "device:" + device_id
        evaluation = "false"
        if self.r.exists(device_key_pattern + ":measure") == 1:
            measure = self.r.get(device_key_pattern + ":measure")
            last_time_on_str = self.r.get(device_key_pattern + ":last_time_on")
            last_time_off_str = self.r.get(device_key_pattern + ":last_time_off")
            time_stop_value_str = self.r.get(key_pattern + ":time_stop_value")
            time_start_value_str = self.r.get(key_pattern + ":time_start_value")
            if last_time_on_str == "" or last_time_off_str == "" or time_stop_value_str == "-" or time_start_value_str == "-":
                return "true"
            last_time_on = datetime.strptime(last_time_on_str, '%H:%M')
            last_time_off = datetime.strptime(last_time_off_str, '%H:%M')
            if measure == "on":
                delta_last_off = (last_time_on - last_time_off).total_seconds()
                time_stop_value = datetime.strptime(time_stop_value_str, '%H:%M').time()
                date_stop_value = int(self.r.get(key_pattern + ":date_stop_value")) * 24 * 60
                stop_interval = (time_stop_value.hour * 60) + time_stop_value.minute + date_stop_value
                evaluation = "true"
                if stop_interval < delta_last_off:
                    evaluation = "false"
            elif measure == "off":
                delta_last_on = (last_time_off - last_time_on).total_seconds()
                time_start_value = datetime.strptime(time_start_value_str, '%H:%M').time()
                date_start_value = int(self.r.get(key_pattern + ":date_start_value")) * 24 * 60
                start_inteval = (time_start_value.hour * 60) + time_start_value.minute + date_start_value
                evaluation = "false"
                if start_inteval < delta_last_on:
                    evaluation = "true"
        return evaluation



from .WaterLevelAntecedentDTO import WaterLevelAntecedent
from datetime import datetime


class WaterLevelAntecedentFunction(object):
    def __init__(self, redis):
        self.r = redis

    def get_antecedent(self, user_id, rule_id, device_id):
        try:
            antecedent = WaterLevelAntecedent()
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            antecedent.device_id = device_id
            antecedent.device_name = self.r.get("device:" + device_id + ":name")
            antecedent.measure = self.r.get(key_pattern + ":measure")
            antecedent.absolute_measure = self.r.get(key_pattern + ":absolute_measure")
            antecedent.condition_measure = self.r.get(key_pattern + ":condition_measure")
            antecedent.start_value = self.r.get(key_pattern + ":start_value")
            antecedent.stop_value = self.r.get(key_pattern + ":stop_value")
            antecedent.evaluation = self.r.get(key_pattern + ":evaluation")
            antecedent.last_time_on = self.r.get(key_pattern + ":last_time_on")
            antecedent.last_time_off = self.r.get(key_pattern + ":last_time_off")
            antecedent.last_date_on = self.r.get(key_pattern + ":last_date_on")
            antecedent.last_date_off = self.r.get(key_pattern + ":last_date_off")
            return antecedent
        except Exception as error:
            print(repr(error))
            return "error"

    def get_antecedent_slim(self, user_id, rule_id, device_id):
        try:
            antecedent = WaterLevelAntecedent()
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            antecedent.device_id = device_id
            antecedent.device_name = self.r.get("device:" + device_id + ":name")
            antecedent.evaluation = self.r.get(key_pattern + ":evaluation")
            return antecedent
        except Exception as error:
            print(repr(error))
            return "error"

    def delete_antecedent(self, user_id, rule_id, device_id):
        try:
            self.r.lrem("device:" + device_id + ":rules", rule_id)
            self.r.lrem("user:" + user_id + ":rule:" + rule_id + ":device_antecedents", device_id)
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            self.r.delete(key_pattern + ":measure")
            self.r.delete(key_pattern + ":condition_measure")
            self.r.delete(key_pattern + ":start_value")
            self.r.delete(key_pattern + ":stop_value")
            self.r.delete(key_pattern + ":evaluation")
            self.r.delete(key_pattern + ":last_time_on")
            self.r.delete(key_pattern + ":last_time_off")
            self.r.delete(key_pattern + ":last_date_on")
            self.r.delete(key_pattern + ":last_date_off")
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"

    def add_antecedent(self, user_id, rule_id, device_id):
        try:
            device_antecedents = self.r.lrange("user:" + user_id + ":rule:" + rule_id + ":device_antecedents")
            result = "false"
            if device_id not in device_antecedents:
                antecedent = WaterLevelAntecedent()
                self.r.rpush("device:" + device_id + ":rules", rule_id)
                self.r.rpush("user:" + user_id + ":rule:" + rule_id + ":device_antecedents", device_id)
                key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
                self.r.set(key_pattern + ":evaluation", antecedent.evaluation)
                self.r.set(key_pattern + ":condition_measure", antecedent.condition_measure)
                self.r.set(key_pattern + ":start_value", antecedent.start_value)
                self.r.set(key_pattern + ":stop_value", antecedent.stop_value)
                result = "true"
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def update_antecedent(self, user_id, rule_id, new_antecedent):
        try:
            antecedent = WaterLevelAntecedent()
            antecedent.antecedent_mapping(new_antecedent)
            device_antecedents = self.r.lrange("user:" + user_id + ":rule:" + rule_id + ":device_antecedents")
            result = "false"
            if antecedent.device_id in device_antecedents:
                key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + antecedent.device_id
                self.r.set(key_pattern + ":condition_measure", antecedent.condition_measure)
                self.r.set(key_pattern + ":start_value", antecedent.start_value)
                self.r.set(key_pattern + ":stop_value", antecedent.stop_value)
                result = "true"
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def antecedent_evaluation(self, user_id, rule_id, device_id, measure):
        try:
            evaluation = "false"
            if self.r.exists("device:" + device_id + ":measure") == 1:
                evaluation = self.measure_evaluation(user_id, rule_id, device_id, measure)
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            old_evaluation = self.r.get(key_pattern + ":evaluation")
            trigger = "false"
            if evaluation != old_evaluation:
                self.r.set(key_pattern + ":evaluation", evaluation)
                trigger = "true"
            return trigger
        except Exception as error:
            print(repr(error))
            return "error"

    def measure_evaluation(self, user_id, rule_id, device_id, measure):
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        evaluation = "false"
        old_evaluation = self.r.get(key_pattern + ":evaluation")
        condition = self.r.get(key_pattern + ":condition_measure")
        start_value = self.r.get(key_pattern + ":start_value")
        stop_value = self.r.get(key_pattern + ":stop_value")
        if start_value == "//" or stop_value == "//":
            return "true"
        if condition == "between":
            if int(start_value) <= int(measure) < int(stop_value):
                evaluation = "true"
        elif condition == ">":
            if int(measure) > int(start_value):
                evaluation = "true"
        elif condition == "<":
            if int(measure) < int(start_value):
                evaluation = "true"
        elif condition == "isteresi":
            if int(measure) <= int(start_value):
                evaluation = "true"
            if old_evaluation == "true" and int(measure) <= int(stop_value):
                evaluation = "true"
        return evaluation



from .FolderDTO import FolderDto


class FolderFunction(object):
    def __init__(self, redis):
        self.r = redis

    def get_folder(self, user_id, folder_id):
        try:
            folder = FolderDto()
            key_pattern = "user:" + user_id + ":folder:" + folder_id
            folder.folder_id = folder_id
            folder.folder_name = self.r.get(key_pattern+":name")
            folder.rules = self.r.lrange(key_pattern+":rules")
            return folder
        except Exception as error:
            print(repr(error))
            return "error"

    def create_folder(self, user_id, folder_name):
        try:
            folder_id = str(self.r.incr("user:" + user_id + ":folder:counter"))
            self.r.rpush("user:" + user_id + ":folders", folder_id)
            self.r.set("user:" + user_id + ":folder:" + folder_id + ":name", folder_name)
            folder = FolderDto()
            folder.folder_id = folder_id
            folder.folder_name = folder_name
            return folder
        except Exception as error:
            print(repr(error))
            return "error"

    def update_folder(self, user_id, folder):
        try:
            key_pattern = "user:" + user_id + ":folder:" + folder.folder_id
            self.r.set(key_pattern+":name", folder.name)
            rules = self.r.lrange(key_pattern+":rules")
            for rule in rules:
                self.r.lrem(key_pattern+":rules", rule)
            for rule in folder.rules:
                self.r.rpush(key_pattern+":rules", rule)
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"

    def delete_folder(self, user_id, folder_id):
        try:
            self.r.lrem("user:" + user_id + ":folders", folder_id)
            self.r.delete("user:" + user_id + ":folder:" + folder_id + ":name")
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"

    def add_rule(self, user_id, folder_id, rule_id):
        try:
            self.r.rpush("user:"+user_id+":folder:"+folder_id+":rules", rule_id)
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"

    def delete_rule(self, user_id, folder_id, rule_id):
        try:
            self.r.lrem("user:"+user_id+":folder:"+folder_id+":rules", rule_id)
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"



class SwitchAntecedent(object):
    def __init__(self):
        self.device_id = ""
        self.device_name = ""
        self.condition = "between"
        self.last_time_on = ""
        self.last_date_on = ""
        self.last_time_off = ""
        self.last_date_off = ""
        self.time_start_value = "-"
        self.date_start_value = "0"
        self.time_stop_value = "-"
        self.date_stop_value = "0"
        self.evaluation = "false"

    def antecedent_mapping(self, antecedent):
        self.device_id = antecedent["device_id"]
        self.device_name = antecedent["device_name"]
        self.time_start_value = antecedent["time_start_value"]
        self.date_start_value = antecedent["date_start_value"]
        self.time_stop_value = antecedent["time_stop_value"]
        self.date_stop_value = antecedent["date_stop_value"]



from .TimerAntecedentDTO import TimerAntecedent
from datetime import datetime


class TimerAntecedentFunction(object):
    def __init__(self, redis):
        self.r = redis

    def get_antecedent(self, user_id, rule_id, device_id):
        try:
            antecedent = TimerAntecedent()
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            antecedent.device_id = device_id
            antecedent.device_name = self.r.get("device:" + device_id + ":name")
            antecedent.day_start_value = self.r.lrange(key_pattern + ":day_start_value")
            antecedent.time_start_value = self.r.get(key_pattern + ":time_start_value")
            antecedent.time_stop_value = self.r.get(key_pattern + ":time_stop_value")
            antecedent.check_time = self.r.get(key_pattern + ":check_time")
            antecedent.check_date = self.r.get(key_pattern + ":check_date")
            antecedent.evaluation = self.r.get(key_pattern + ":evaluation")
            antecedent.measure_time = datetime.now().strftime("%H:%M")
            antecedent.measure_day = str(datetime.today().weekday())
            return antecedent
        except Exception as error:
            print(repr(error))
            return "error"

    def get_antecedent_slim(self, user_id, rule_id, device_id):
        try:
            antecedent = TimerAntecedent()
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            antecedent.device_id = device_id
            antecedent.device_name = self.r.get("device:" + device_id + ":name")
            antecedent.evaluation = self.r.get(key_pattern + ":evaluation")
            return antecedent
        except Exception as error:
            print(repr(error))
            return "error"

    def delete_antecedent(self, user_id, rule_id, device_id):
        try:
            self.r.lrem("device:" + device_id + ":rules", rule_id)
            self.r.lrem("user:" + user_id + ":rule:" + rule_id + ":device_antecedents", device_id)
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            if self.r.exists(key_pattern + ":day_start_value") == 1:
                days = self.r.lrange(key_pattern + ":day_start_value")
                for day in days:
                    self.r.lrem(key_pattern + ":day_start_value", day)
            self.r.delete(key_pattern + ":time_start_value")
            self.r.delete(key_pattern + ":time_stop_value")
            self.r.delete(key_pattern + ":check_time")
            self.r.delete(key_pattern + ":check_date")
            self.r.delete(key_pattern + ":evaluation")
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"

    def add_antecedent(self, user_id, rule_id, device_id):
        try:
            device_antecedents = self.r.lrange("user:" + user_id + ":rule:" + rule_id + ":device_antecedents")
            result = "false"
            if device_id not in device_antecedents:
                antecedent = TimerAntecedent()
                self.r.rpush("device:" + device_id + ":rules", rule_id)
                self.r.rpush("user:" + user_id + ":rule:" + rule_id + ":device_antecedents", device_id)
                key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
                self.r.set(key_pattern + ":evaluation", antecedent.evaluation)
                self.r.set(key_pattern + ":time_start_value", antecedent.time_start_value)
                self.r.set(key_pattern + ":time_stop_value", antecedent.time_stop_value)
                self.r.set(key_pattern + ":check_time", antecedent.check_time)
                self.r.set(key_pattern + ":check_date", antecedent.check_date)
                result = "true"
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def update_antecedent(self, user_id, rule_id, new_antecedent):
        try:
            antecedent = TimerAntecedent()
            antecedent.antecedent_mapping(new_antecedent)
            device_antecedents = self.r.lrange("user:" + user_id + ":rule:" + rule_id + ":device_antecedents")
            result = "false"
            if antecedent.device_id in device_antecedents:
                key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + antecedent.device_id
                if self.r.exists(key_pattern + ":day_start_value") == 1:
                    days = self.r.lrange(key_pattern + ":day_start_value")
                    for day in days:
                        self.r.lrem(key_pattern + ":day_start_value", day)
                if len(antecedent.day_start_value) > 0:
                    for day in antecedent.day_start_value:
                        self.r.rpush(key_pattern + ":day_start_value", day)
                self.r.set(key_pattern + ":time_start_value", antecedent.time_start_value)
                self.r.set(key_pattern + ":time_stop_value", antecedent.time_stop_value)
                self.r.set(key_pattern + ":check_time", antecedent.check_time)
                self.r.set(key_pattern + ":check_date", antecedent.check_date)
                result = "true"
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def antecedent_evaluation(self, user_id, device_id, rule_id):
        try:
            time_evaluation = self.evaluate_time(user_id, device_id, rule_id)
            date_evaluation = self.evaluate_date(user_id, device_id, rule_id)
            evaluation = "false"
            if time_evaluation == "true" and date_evaluation == "true":
                evaluation = "true"
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            old_evaluation = self.r.get(key_pattern + ":evaluation")
            trigger = "false"
            if evaluation != old_evaluation:
                self.r.set(key_pattern + ":evaluation", evaluation)
                trigger = "true"
            return trigger
        except Exception as error:
            print(repr(error))
            return "error"

    def evaluate_time(self, user_id, device_id, rule_id):
        try:
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            check_time = self.r.get(key_pattern + ":check_time")
            time_start_value_str = self.r.get(key_pattern + ":time_start_value")
            time_stop_value_str = self.r.get(key_pattern + ":time_stop_value")
            evaluation = "true"
            if check_time == "true" and time_start_value_str != "" and time_stop_value_str != "":
                current_time_str = datetime.now().strftime("%H:%M")
                current_time = datetime.strptime(current_time_str, '%H:%M').time()
                time_start_value = datetime.strptime(time_start_value_str, '%H:%M').time()
                time_stop_value = datetime.strptime(time_stop_value_str, '%H:%M').time()
                evaluation = "false"
                if time_start_value <= current_time < time_stop_value:
                    evaluation = "true"
            return evaluation
        except Exception as error:
            print(repr(error))
            return "error"

    def evaluate_date(self, user_id, device_id, rule_id):
        try:
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
            check_date = self.r.get(key_pattern + ":check_date")
            day_start_value = self.r.lrange(key_pattern + ":day_start_value")
            evaluation = "true"
            if check_date == "true" and self.r.exists(key_pattern + ":day_start_value") == 1 and len(
                    day_start_value) > 0:
                current_day = str(datetime.today().weekday())
                evaluation = "false"
                if current_day in day_start_value:
                    evaluation = "true"
            return evaluation
        except Exception as error:
            print(repr(error))
            return "error"
